
/**
 * Title:        ASCII File Resource Adapter<p>
 * Description:  This project is about building a JCA 1.0 compatible
 * resource adapter as part of the book on developing
 * JCA adapters.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iConexio.jca.asciiFileRA;

import javax.resource.cci.*;
import javax.resource.NotSupportedException;
import java.util.Map;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.IllegalStateException;
import javax.resource.spi.*;

/** This class represents the connection handle between the ASCII file
 * adapter and the application component or client.
 * The ASCII file adapter can be part of a local transaction however the entire
 * ASCII file will be parsed as a single step with a begin and an end as
 * transaction demarcations
 */
public class ASCIIFileConnection implements javax.resource.cci.Connection {

  /** Each connection handle has a reference to the managed connection
   * that interfaces or represents the actual ASCII file.
   */
  public ASCIIFileConnection(ASCIIFileManagedConnection mc)
  {
    this.mc = mc;
  }
  ASCIIFileManagedConnection getManagedConnection()
  {
    return(mc);
  }

  public String getASCIIFileName() throws javax.resource.ResourceException
  {
    if (mc == null)
    {
       throw new ResourceException("Connection is already closed");
    }
    return(mc.getASCIIFileName());
  }

  public javax.resource.cci.LocalTransaction getLocalTransaction() throws
                                javax.resource.ResourceException
  {
    // Since the ASCII file itself does not have a any support for transactions
    // the adapter only supports local transactions for this adapter.
    return(new AFLocalTransactionImpl(mc));
  }
  public void setAutoCommit(boolean autoCommit) throws ResourceException
  {
    throw new ResourceException(" Auto Commit is not supported");
  }

  public boolean getAutoCommit() throws ResourceException
  {
    return false;
  }

  /** This adapter does not use the ResultSetInfo class to return data
   * ResultSetInfo is more useful when accessing a database rather then
   * a flat file. The Interaction class of this adapter returns a vector of
   * customer records instead
   */
  public javax.resource.cci.ResultSetInfo getResultSetInfo() throws
                                javax.resource.ResourceException {
    NotSupportedException e = new
         NotSupportedException("Result Set Info Not Supported");
    throw e;
  }

  public void close() throws ResourceException {
      if (mc == null) return;  // already closed
      mc.removeAFConnection(this); // Remove this connection from the set of
                                   // connections in the managed connection
                                   // instance
      mc = null;
      connectionDestroyed = true;
  }

  public javax.resource.cci.ConnectionMetaData getMetaData() throws
                                javax.resource.ResourceException {
    ASCIIFileMetaData md = new ASCIIFileMetaData();
    return(md);
  }

  /** The createInteraction method returns an instance of ASCIIFileInteraction
   * This object contains services or methods that provide the capability of
   * parsing customer data files in ASCII format.
   */
  public javax.resource.cci.Interaction createInteraction() throws
                                javax.resource.ResourceException
  {
    ASCIIFileInteraction aint = new ASCIIFileInteraction(this);
    return(aint);
  }

  // This method is called by the application server when needed.
  // The application server wants the connection to be associated with a
  // new or different managed connection instance.
  void associateConnection(ASCIIFileManagedConnection newMc)
      throws ResourceException
  {
      if (mc == null)
      {
         // Then the connection was closed and therefore no need to
         // associate it with the new or different managed connection
         throw new IllegalStateException("Connection is either closed or "+
                                         "invalid");
      }
      else
      {
        // dissociate handle with current managed connection
        mc.removeAFConnection(this);
        // associate handle with new managed connection
        newMc.addAFConnection(this);
        mc = newMc;
      }
  }

  private void checkIfDestroyed() throws ResourceException {
      if (connectionDestroyed) {
          throw new IllegalStateException("Managed connection is closed");
      }
  }

  public void invalidate()
  {
    mc = null;
  }

  // private member variables
  private ASCIIFileManagedConnection mc; // Represents the physical connection
                                         // to the ASCII data file
  private boolean connectionDestroyed; // Flag to indicate that the connection
                                          // is destroyed.
}


